/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Dict
 *
 * The Initial Developer of the Original Code is David Costanzo.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function DictServer (name, port, description) {
    this.name        = name;
    this.port        = port;
    this.description = description;
}

// The preferences that we handle
const PREFNAME_DEFAULTSERVERNAME = "extensions.dict.defaultservername";
const PREFNAME_DEFAULTSERVERPORT = "extensions.dict.defaultserverport";


// The list of valid DICT servers
var DICT_SERVERS = [
    new DictServer(
        'dega.cs.unc.edu',
        2628,
        'To and from lojban and a number of other languages.'),
    new DictServer(
        'dict.arabeyes.org',
        2628,
        'English-Arabic, extensible'),
    new DictServer(
        'dict.netground.com',
        2628,
        'Standard and freedict databases, GCIDE, Netground ISW\'s Dictionary'),
    new DictServer(
        'dict.org',
        2628,
        'The canonical dictd server, contains mainly databases related to English language'),
    new DictServer(
        'dict.rucus.ru.ac.za',
        2628,
        'Standard dictd server with eleven English databases.'),
    new DictServer(
        'dict.tu-chemnitz.de',
        2628,
        'Standard DICT server with 9 databases (devils, elements, foldoc, gazetteer, gcide, jargon, vera, web1913, Eastons, Hitchcock\'s, CIA World Factbook 1995, CIA World Factbook 2002 and wn)'),
    new DictServer(
        'dict.tugraz.at',
        2628,
        'standard and freedict databases'),
    new DictServer(
        'dict.uni-leipzig.de',
        2628,
        'Khasi-German, German-Khasi (1000 headwords), English-Hindi (25645 headwords)'),
    new DictServer(
        'kassiopeia.juls.savba.sk',
        2628,
        'Many different dictionaries (en, slovak, hu, ru, bulgarian, serbian, german, latin, ...)'),
    new DictServer(
        'localhost',
        2628,
        'The DICT server that is running on your local machine.'),
    new DictServer(
        'madison.onespeeddave.com',
        2628,
        'Web interface offers cross-referenced definitions from 11 standard databases, automatic spelling correction, regular expressions, and also supports queries via http://[query].dict.die.net.'),
    new DictServer(
        'mova.org',
        2628,
        'A collection of Slavic dictionaries'),
    new DictServer(
        'slovnik.zcu.cz',
        2628,
        'chinese-english, moby thesaurus, standard databases'),
    new DictServer(
        'test.dict.org',
        2628,
        'Standard databases'),
    new DictServer(
        'www.okpos.com',
        2628,
        'Standard databases'),
];

// handler for when a new DICT server is selected in the list box
function newDictServerSelected() {
    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);


    var descriptionElement = document.getElementById("description"); 

    // remove any exising description
    while (descriptionElement.firstChild) {
        descriptionElement.removeChild(descriptionElement.firstChild);
    }

    var listbox = document.getElementById("dictd-servers");

    var serverID = listbox.selectedIndex;

    // Update the description text to be match the description of the
    // newly selected server.  Also update the hidden preference elements.
    if (serverID < 0 || DICT_SERVERS.length <= serverID) {

        // The selected server is not in our stock list of suggested
        // servers.  Do not choose one by default and modify the user's
        // preferences, because the user may have hacked their prefs.js
        // file to point to a server that is not on our hard-coded list.

        // add the new description
        descriptionText = 'Please select a server';
        var descriptionTextNode = document.createTextNode(descriptionText);
        descriptionElement.appendChild(descriptionTextNode);
    }
    else {
        var server = DICT_SERVERS[serverID];

        // update the preferences in our hidden preference elements
        var defaultServerName = document.getElementById('defaultservername');
        defaultServerName.setAttribute('value', server.name);

        var defaultServerPort = document.getElementById('defaultserverport');
        defaultServerPort.setAttribute('value', server.port);

        // add the new description
        var descriptionTextNode = document.createTextNode(server.description);
        descriptionElement.appendChild(descriptionTextNode);
    }
}

function dictDefaultPreferencesValuesSet() {
    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

    try {
        pref.getCharPref(PREFNAME_DEFAULTSERVERNAME);
        pref.getIntPref(PREFNAME_DEFAULTSERVERPORT);
    }
    catch (e) {
        // something is wrong with the preferences,
        // restore them to a known good state.
        try {
            pref.setCharPref(PREFNAME_DEFAULTSERVERNAME, 'dict.org');
            pref.setIntPref(PREFNAME_DEFAULTSERVERPORT,  2628);
        }
        catch (e) {
        }
    }
}


// This gets called automatically from Mozilla Application Suite
function Startup() {
    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

    // set default prefs
    dictDefaultPreferencesValuesSet();

    // set the text to include the version number
    //var msg = document.getElementById("bundle_dict").getString("dict-pref-version");

    var listbox = document.getElementById("dictd-servers");

    for (dictserver in DICT_SERVERS) {

        // append this server to the list box
        var listitem = document.createElement('listitem');
        listitem.setAttribute('value', dictserver);
        listitem.setAttribute('label', DICT_SERVERS[dictserver].name);

        listbox.appendChild(listitem);
    }


    // Try to find an index in DICT_SERVERS that corresponds to the selected 
    // default server so that we can select it.
    var defaultServerIndex = 0;
    try {
        var defaultServerName = pref.getCharPref(PREFNAME_DEFAULTSERVERNAME);
        var defaultServerPort = pref.getIntPref(PREFNAME_DEFAULTSERVERPORT);

        var currentServerIndex;
        for (currentServerIndex in DICT_SERVERS) {
            if (DICT_SERVERS[currentServerIndex].name == defaultServerName &&
                DICT_SERVERS[currentServerIndex].port == defaultServerPort) {
                defaultServerIndex = currentServerIndex;
                break;
            }
        }
    }
    catch (e) {
    }

    listbox.selectedIndex = defaultServerIndex;
}



function dictPreferencesInit() {
    // Manually call the Startup since Mozilla Firebird doesn't call it
    Startup();
}


function dictPreferencesSave() {

    // Manually save prefs; Mozilla Firebird doesn't do it
    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

    try {
        var defaultServerName = document.getElementById("defaultservername").value;

        pref.setCharPref(
            defaultServerName.getAttribute("prefstring"),
            defaultServerName.value);


        var defaultServerPort = document.getElementById("defaultserverport").value;

        pref.setIntPref(
            defaultServerPort.getAttribute("prefstring"),
            defaultServerPort.value);
    } 
    catch(e) {
    }
}
